@@ -440,21 +440,25 @@ and mkwithc =
               error loc msg ]
       | None → error loc "no with type := in this ocaml version" ] ]
 
-and mkvalue_desc ~{item_attributes} vn t p =
+and mkvalue_desc ~{item_attributes} vn (tyvars, t) p =
+  let t = match tyvars with [ [] -> t | l ->
+    let loc = loc_of_ctyp t in
+    TyPol loc <:vala< l >> t ] in
  ocaml_value_description ~{item_attributes=item_attributes} vn (ctyp t) p
 
-and sumbranch_ctyp ?{priv=False} loc l rto =
+and sumbranch_ctyp ?{priv=False} loc tyvars l rto =
     let rto = option_map ctyp rto in
     match l with [
       [TyRec loc ltl] -> do {
+        assert ([] = tyvars) ;
         (Right (mktrecord (uv ltl) priv), rto)
       }
     | [TyRec _ _ :: _] -> error loc "only ONE record type allowed here"
-    | l -> (Left (List.map ctyp l), rto)
+    | l -> (Left (tyvars, List.map ctyp l), rto)
     ]
 
-and conv_constructor priv (loc, c, tl, rto, alg_attrs) =
-    (conv_con (uv c), sumbranch_ctyp ~{priv=priv} loc (uv tl) (uv rto),
+and conv_constructor priv (loc, c, tyvars, tl, rto, alg_attrs) =
+    (conv_con (uv c), sumbranch_ctyp ~{priv=priv} loc (uv tyvars) (uv tl) (uv rto),
      mkloc loc, uv_alg_attributes alg_attrs)
 
 and mktvariant loc ctl priv =
@@ -638,8 +642,8 @@ and extension_constructor ec = match ec with [
       EcTuple loc gc ->
       let (n, tl, _, alg_attrs) = conv_constructor False gc in
       match tl with [
-        (Left x,y) -> 
-          ocaml_ec_tuple ~{alg_attributes=alg_attrs} (mkloc loc) n (x,y)
+        (Left (tyvars, x),y) -> 
+          ocaml_ec_tuple ~{alg_attributes=alg_attrs} (mkloc loc) n tyvars (x,y)
       | (Right x, y) -> do {
           ocaml_ec_record ~{alg_attributes=alg_attrs} (mkloc loc) n (x,y)
         }
@@ -1275,9 +1279,9 @@ and sig_item s l =
       [mksig loc
          (ocaml_psig_exception ~{alg_attributes=alg_attrs} ~{item_attributes=uv_item_attributes item_attrs} (mkloc loc) n tl) ::
        l]
-  | SgExt loc n t p attrs →
+  | SgExt loc n ls t p attrs →
       let vn = uv n in
-      [mksig loc (ocaml_psig_value vn (mkvalue_desc ~{item_attributes=uv_item_attributes attrs} vn t (uv p))) :: l]
+      [mksig loc (ocaml_psig_value vn (mkvalue_desc ~{item_attributes=uv_item_attributes attrs} vn (uv ls, t) (uv p))) :: l]
   | SgInc loc mt attrs →
       [mksig loc (ocaml_psig_include ~{item_attributes=uv_item_attributes attrs} (mkloc loc) (module_type mt)) :: l]
   | SgMod loc rf ntl →
@@ -1348,7 +1352,7 @@ and sig_item s l =
         (fun () → List.fold_right (fun (si, _) → sig_item si) (uv sl) l) ()
   | SgVal loc n t attrs →
       let vn = uv n in
-      [mksig loc (ocaml_psig_value vn (mkvalue_desc ~{item_attributes=uv_item_attributes attrs} vn t [])) :: l]
+      [mksig loc (ocaml_psig_value vn (mkvalue_desc ~{item_attributes=uv_item_attributes attrs} vn ([], t) [])) :: l]
   | SgXtr loc _ _ → error loc "bad ast SgXtr"
   | SgFlAtt loc float_attr →
       [mksig loc
@@ -1434,9 +1438,9 @@ and str_item s l =
       [mkstr loc si :: l]
     ]
   | StExp loc e attrs → [mkstr loc (ocaml_pstr_eval ~{item_attributes=uv_item_attributes attrs} (expr e)) :: l]
-  | StExt loc n t p attrs →
+  | StExt loc n ls t p attrs →
       let vn = uv n in
-      [mkstr loc (ocaml_pstr_primitive vn (mkvalue_desc ~{item_attributes=uv_item_attributes attrs} vn t (uv p))) :: l]
+      [mkstr loc (ocaml_pstr_primitive vn (mkvalue_desc ~{item_attributes=uv_item_attributes attrs} vn (uv ls, t) (uv p))) :: l]
   | StInc loc me attrs →
       match ocaml_pstr_include with
       [ Some pstr_include →
@@ -1788,6 +1792,10 @@ value directive loc =
       match ocaml_pdir_bool with
       [ Some pdir_bool → pdir_bool False
       | None → error loc "no such kind of directive in this ocaml version" ]
+  | <:expr< $longid:li$ >> -> Pdir_ident (longid_long_id li)
+  | <:expr< $longid:li$ >> -> Pdir_ident (longid_long_id li)
+  | <:expr< $longid:li$ . $lid:id$ >> -> Pdir_ident (longid_lident_long_id (Some <:vala< li >>,<:vala< id >>))
+  | <:expr<  $lid:id$ >> -> Pdir_ident (Lident id)
   | e →
       let sl =
         loop e where rec loop =
